@@ -4,7 +4,7 @@ max_rho = []
 max_pos = []
 times = []
 for i in range(30):
-    pf = lagos.EnzoStaticOutput("my_output%04i" % (i))
+    pf = load("my_output%04i" % (i))
     v, c = pf.h.find_max("Density")
     max_rho.append(v)
     max_pos.append(c)
