@@ -51,12 +51,14 @@ ytcfgDefaults = {
         'pluginfilename':'my_plugins.py',
         },
     "yt":{
+        'time_functions': 'False',
         'LogFile': 'False',
         'LogFileName': 'yt.log',
         'suppressStreamLogging': 'False',
         'LogLevel': '20',
         'timefunctions':'False',
         'inGui':'False',
+        'inline':'False',
         '__parallel':'False',
         '__parallel_rank':'0',
         '__parallel_size':'1',
