@@ -101,8 +101,10 @@ class HaloMassFcn(lagos.ParallelAnalysisInterface):
             self.hubble0 = self.pf['CosmologyHubbleConstantNow']
             self.this_redshift = self.pf['CosmologyCurrentRedshift']
             self.read_haloes()
-            self.log_mass_min = math.log10(min(self.haloes))
-            self.log_mass_max = math.log10(max(self.haloes))
+            if self.log_mass_min == None:
+                self.log_mass_min = math.log10(min(self.haloes))
+            if self.log_mass_max == None:
+                self.log_mass_max = math.log10(max(self.haloes))
 
         # Input error check.
         if self.mode == 'single':
@@ -176,8 +178,9 @@ class HaloMassFcn(lagos.ParallelAnalysisInterface):
         """
         mylog.info("Reading halo masses from %s" % self.halo_file)
         f = open(self.halo_file,'r')
-        line = f.readline() # burn the top header line.
         line = f.readline()
+        while line[0] == '#':
+            line = f.readline()
         self.haloes = []
         while line:
             line = line.split()
@@ -201,7 +204,7 @@ class HaloMassFcn(lagos.ParallelAnalysisInterface):
             dis[self.num_sigma_bins-i-3] += dis[self.num_sigma_bins-i-2]
             if i == (self.num_sigma_bins - 3): break
 
-        self.dis = dis / self.pf['CosmologyComovingBoxSize']**3.0
+        self.dis = dis  / self.pf['CosmologyComovingBoxSize']**3.0 * self.hubble0**3.0
 
     def sigmaM(self):
         """
@@ -392,7 +395,7 @@ class HaloMassFcn(lagos.ParallelAnalysisInterface):
         
         if self.fitting_function==1:
             # Press-Schechter (This form from Jenkins et al. 2001, MNRAS 321, 372-384, eqtn. 5)
-            thismult = sqrt(2.0/math.pi) * nu * exp(-0.5*nu*nu);
+            thismult = math.sqrt(2.0/math.pi) * nu * math.exp(-0.5*nu*nu);
         
         elif self.fitting_function==2:
             # Jenkins et al. 2001, MNRAS 321, 372-384, eqtn. 9
