@@ -7,4 +7,6 @@ def configuration(parent_package='',top_path=None):
     config.make_config_py() # installs __config__.py
     config.make_svn_version_py()
     config.add_subpackage("lightcone")
+    config.add_subpackage("volume_rendering")
+    config.add_subpackage("kdtree")
     return config
