@@ -34,6 +34,10 @@
 
 #include "numpy/ndarrayobject.h"
 
+#ifndef npy_float128
+#define npy_float128 npy_longdouble
+#endif
+
 
 static PyObject *_hdf5ReadError;
 herr_t iterate_dataset(hid_t loc_id, const char *name, void *nodelist);
@@ -1280,14 +1284,14 @@ int count_particles_region_FLOAT(particle_validation *data)
         } else if ( (tempx > vdata->left_edge[0]) && (tempx > vdata->right_edge[0]) ) {
           tempx -= vdata->period[0];
         }
-        if ( (tempy < vdata->left_edge[1]) && (tempx < vdata->right_edge[1]) ) {
+        if ( (tempy < vdata->left_edge[1]) && (tempy < vdata->right_edge[1]) ) {
           tempy += vdata->period[1];
-        } else if ( (tempy > vdata->left_edge[1]) && (tempx > vdata->right_edge[1]) ) {
+        } else if ( (tempy > vdata->left_edge[1]) && (tempy > vdata->right_edge[1]) ) {
           tempy -= vdata->period[1];
         }
-        if ( (tempz < vdata->left_edge[2]) && (tempx < vdata->right_edge[2]) ) {
+        if ( (tempz < vdata->left_edge[2]) && (tempz < vdata->right_edge[2]) ) {
           tempz += vdata->period[2];
-        } else if ( (tempz > vdata->left_edge[2]) && (tempx > vdata->right_edge[2]) ) {
+        } else if ( (tempz > vdata->left_edge[2]) && (tempz > vdata->right_edge[2]) ) {
           tempz -= vdata->period[2];
         }
         if (   (tempx >= vdata->left_edge[0])
@@ -1350,14 +1354,14 @@ int count_particles_region_DOUBLE(particle_validation *data)
         } else if ( (tempx > vdata->left_edge[0]) && (tempx > vdata->right_edge[0]) ) {
           tempx -= vdata->period[0];
         }
-        if ( (tempy < vdata->left_edge[1]) && (tempx < vdata->right_edge[1]) ) {
+        if ( (tempy < vdata->left_edge[1]) && (tempy < vdata->right_edge[1]) ) {
           tempy += vdata->period[1];
-        } else if ( (tempy > vdata->left_edge[1]) && (tempx > vdata->right_edge[1]) ) {
+        } else if ( (tempy > vdata->left_edge[1]) && (tempy > vdata->right_edge[1]) ) {
           tempy -= vdata->period[1];
         }
-        if ( (tempz < vdata->left_edge[2]) && (tempx < vdata->right_edge[2]) ) {
+        if ( (tempz < vdata->left_edge[2]) && (tempz < vdata->right_edge[2]) ) {
           tempz += vdata->period[2];
-        } else if ( (tempz > vdata->left_edge[2]) && (tempx > vdata->right_edge[2]) ) {
+        } else if ( (tempz > vdata->left_edge[2]) && (tempz > vdata->right_edge[2]) ) {
           tempz -= vdata->period[2];
         }
         if (   (tempx >= vdata->left_edge[0])
@@ -1420,14 +1424,14 @@ int count_particles_region_LONGDOUBLE(particle_validation *data)
         } else if ( (tempx > vdata->left_edge[0]) && (tempx > vdata->right_edge[0]) ) {
           tempx -= vdata->period[0];
         }
-        if ( (tempy < vdata->left_edge[1]) && (tempx < vdata->right_edge[1]) ) {
+        if ( (tempy < vdata->left_edge[1]) && (tempy < vdata->right_edge[1]) ) {
           tempy += vdata->period[1];
-        } else if ( (tempy > vdata->left_edge[1]) && (tempx > vdata->right_edge[1]) ) {
+        } else if ( (tempy > vdata->left_edge[1]) && (tempy > vdata->right_edge[1]) ) {
           tempy -= vdata->period[1];
         }
-        if ( (tempz < vdata->left_edge[2]) && (tempx < vdata->right_edge[2]) ) {
+        if ( (tempz < vdata->left_edge[2]) && (tempz < vdata->right_edge[2]) ) {
           tempz += vdata->period[2];
-        } else if ( (tempz > vdata->left_edge[2]) && (tempx > vdata->right_edge[2]) ) {
+        } else if ( (tempz > vdata->left_edge[2]) && (tempz > vdata->right_edge[2]) ) {
           tempz -= vdata->period[2];
         }
         if (   (tempx >= vdata->left_edge[0])
