@@ -864,9 +864,428 @@ Py_DataCubeReplace(PyObject *obj, PyObject *args)
     return to_return;
 }
 
+static PyObject *Py_FillRegion(PyObject *obj, PyObject *args)
+{
+    PyObject *oc_data, *og_data,
+             *oc_start, *og_start,
+             *oc_dims, *og_dims, *omask;
+    PyObject *tg_data, *tc_data, *dw_data;
+    oc_data = og_data = oc_start = og_start = oc_dims = og_dims = omask = NULL;
+    tg_data = tc_data = dw_data = NULL;
+    PyArrayObject **g_data, **c_data, *mask,
+                  *g_start, *c_start, *c_dims, *g_dims, *dwa;
+    mask = g_start = c_start = c_dims = g_dims = NULL;
+    g_data = c_data = NULL;
+    int refratio, ll, direction, n;
+    npy_int64 gxs, gys, gzs, gxe, gye, gze;
+    npy_int64 cxs, cys, czs, cxe, cye, cze;
+    npy_int64 ixs, iys, izs, ixe, iye, ize;
+    npy_int64 gxi, gyi, gzi, cxi, cyi, czi;
+    npy_int64 cdx, cdy, cdz;
+    npy_int64 dw[3];
+    int i;
+    npy_int64 ci, cj, ck, ri, rj, rk;
+    int total = 0;
+    void (*to_call)(PyArrayObject* c_data, npy_int64 xc,
+                         npy_int64 yc, npy_int64 zc,
+                    PyArrayObject* g_data, npy_int64 xg,
+                         npy_int64 yg, npy_int64 zg);
+    if (!PyArg_ParseTuple(args, "iOOOOOOOOii",
+            &refratio, &og_start, &oc_start,
+            &oc_data, &og_data,
+            &oc_dims, &og_dims, &omask, &dw_data, &ll, &direction))
+    return PyErr_Format(_dataCubeError,
+            "DataCubeGeneric: Invalid parameters.");
+
+    if (direction == 0) to_call = dcRefine;
+    else if (direction == 1) to_call = dcReplace;
+
+    g_start = (PyArrayObject *) PyArray_FromAny(og_start,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(g_start == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: g_start invalid.");
+      goto _fail;
+    }
+
+    c_start = (PyArrayObject *) PyArray_FromAny(oc_start,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(c_start == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: c_start invalid.");
+      goto _fail;
+    }
+
+    g_dims  = (PyArrayObject *) PyArray_FromAny(og_dims,
+                PyArray_DescrFromType(NPY_INT32), 1, 1, 0, NULL);
+    if(g_dims == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: g_dims invalid.");
+      goto _fail;
+    }
+
+    c_dims  = (PyArrayObject *) PyArray_FromAny(oc_dims,
+                PyArray_DescrFromType(NPY_INT32), 1, 1, 0, NULL);
+    if(c_dims == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: c_dims invalid.");
+      goto _fail;
+    }
+
+    mask    = (PyArrayObject *) PyArray_FromAny(omask,
+                PyArray_DescrFromType(NPY_INT32), 3, 3, 0, NULL);
+    if(mask == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: mask invalid.");
+      goto _fail;
+    }
+
+    dwa     = (PyArrayObject *) PyArray_FromAny(dw_data,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(dwa == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: domain width invalid.");
+      goto _fail;
+    }
+    for (i=0;i<3;i++)dw[i] = *(npy_int64*) PyArray_GETPTR1(dwa, i);
+
+    int n_fields = PyList_Size(oc_data);
+    if(n_fields == 0) {
+      PyErr_Format(_dataCubeError,
+          "CombineGrids: Length zero for c_data is invalid.");
+      goto _fail;
+    }
+    if (!PyList_Check(og_data) || (PyList_Size(og_data) != n_fields)){
+      PyErr_Format(_dataCubeError,
+          "CombineGrids: g_data must be a list of arrays same length as c_data!");
+      goto _fail;
+    }
+
+    c_data = (PyArrayObject**)
+             malloc(sizeof(PyArrayObject*)*n_fields);
+    g_data = (PyArrayObject**)
+             malloc(sizeof(PyArrayObject*)*n_fields);
+    for (n=0;n<n_fields;n++)c_data[n]=g_data[n]=NULL;
+    for (n=0;n<n_fields;n++){
+      tg_data = (PyObject *) PyList_GetItem(og_data, n);
+      /* We set up an array so we only have to do this once 
+         Note that this is an array in the C sense, not the NumPy sense */
+      g_data[n]    = (PyArrayObject *) PyArray_FromAny(tg_data,
+          PyArray_DescrFromType(NPY_FLOAT64), 3, 3,
+          NPY_UPDATEIFCOPY, NULL);
+      if((g_data[n]==NULL) || (g_data[n]->nd != 3)) {
+        PyErr_Format(_dataCubeError,
+            "CombineGrids: Three dimensions required for g_data[%i].",n);
+        goto _fail;
+      }
+      tc_data = (PyObject *) PyList_GetItem(oc_data, n);
+      c_data[n]    = (PyArrayObject *) PyArray_FromAny(tc_data,
+          PyArray_DescrFromType(NPY_FLOAT64), 3, 3,
+          NPY_UPDATEIFCOPY, NULL);
+      if((c_data[n]==NULL) || (c_data[n]->nd != 3)) {
+        PyErr_Format(_dataCubeError,
+            "CombineGrids: Three dimensions required for c_data[%i].",n);
+        goto _fail;
+      }
+    }
+
+    /* g[xyz][se] are the start and end index in integers 
+       of the grid, at its refinement level               */
+    gxs = *(npy_int64 *) PyArray_GETPTR1(og_start, 0);
+    gys = *(npy_int64 *) PyArray_GETPTR1(og_start, 1);
+    gzs = *(npy_int64 *) PyArray_GETPTR1(og_start, 2);
+    gxe = gxs + *(npy_int32 *) PyArray_GETPTR1(og_dims, 0);
+    gye = gys + *(npy_int32 *) PyArray_GETPTR1(og_dims, 1);
+    gze = gzs + *(npy_int32 *) PyArray_GETPTR1(og_dims, 2);
+
+    /* c[xyz][se] are the start and end index in integers
+       of the covering grid, at its refinement level      */
+    cxs = *(npy_int64 *) PyArray_GETPTR1(oc_start, 0);
+    cys = *(npy_int64 *) PyArray_GETPTR1(oc_start, 1);
+    czs = *(npy_int64 *) PyArray_GETPTR1(oc_start, 2);
+
+    /* cd[xyz] are the dimensions of the covering grid */
+    cdx = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 0));
+    cdy = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 1));
+    cdz = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 2));
+    cxe = (cxs + cdx - 1);
+    cye = (cys + cdy - 1);
+    cze = (czs + cdz - 1);
+
+    /* It turns out that C89 doesn't define a mechanism for choosing the sign
+       of the remainder.
+    */
+        //fprintf(stderr, "ci == %d, cxi == %d, dw[0] == %d\n", (int) ci, (int) cxi, (int) dw[0]);
+    for(cxi=cxs;cxi<=cxe;cxi++) {
+        ci = (cxi % dw[0]);
+        ci = (ci < 0) ? ci + dw[0] : ci;
+        if ( ci < gxs*refratio || ci >= gxe*refratio) continue;
+        gxi = floor(ci / refratio) - gxs;
+        for(cyi=cys;cyi<=cye;cyi++) {
+            cj = cyi % dw[1];
+            cj = (cj < 0) ? cj + dw[1] : cj;
+            if ( cj < gys*refratio || cj >= gye*refratio) continue;
+            gyi = floor(cj / refratio) - gys;
+            for(czi=czs;czi<=cze;czi++) {
+                ck = czi % dw[2];
+                ck = (ck < 0) ? ck + dw[2] : ck;
+                if ( ck < gzs*refratio || ck >= gze*refratio) continue;
+                gzi = floor(ck / refratio) - gzs;
+                    if ((ll) || (*(npy_int32*)PyArray_GETPTR3(mask, gxi,gyi,gzi) > 0)) 
+                {
+                for(n=0;n<n_fields;n++){
+                    to_call(c_data[n],
+                        cxi - cxs, cyi - cys, czi - czs,
+                        g_data[n], gxi, gyi, gzi);
+                }
+                total += 1;
+                }
+            }
+        }
+    }
+
+    Py_DECREF(g_start);
+    Py_DECREF(c_start);
+    Py_DECREF(g_dims);
+    Py_DECREF(c_dims);
+    Py_DECREF(mask);
+    for(n=0;n<n_fields;n++) {
+        Py_DECREF(g_data[n]);
+        Py_DECREF(c_data[n]);
+    }
+    free(g_data);
+    free(c_data);
+    PyObject *status = PyInt_FromLong(total);
+    return status;
+
+_fail:
+    Py_XDECREF(g_start);
+    Py_XDECREF(c_start);
+    Py_XDECREF(g_dims);
+    Py_XDECREF(c_dims);
+    Py_XDECREF(mask);
+    if(g_data != NULL)
+      for(n=0;n<n_fields;n++)if(g_data[n]!=NULL){Py_XDECREF(g_data[n]);}
+    if(c_data != NULL)
+      for(n=0;n<n_fields;n++)if(c_data[n]!=NULL){Py_XDECREF(c_data[n]);}
+    if(g_data!=NULL)free(g_data);
+    if(c_data!=NULL)free(c_data);
+    return NULL;
+}
+
+static PyObject *Py_FillBuffer(PyObject *obj, PyObject *args)
+{
+    PyObject *oc_data, *og_data,
+             *oc_start, *og_start,
+             *oc_dims, *og_dims, *omask, *odls;
+    PyObject *tg_data, *tc_data, *dw_data;
+    oc_data = og_data = oc_start = og_start = oc_dims = og_dims = omask = NULL;
+    tg_data = tc_data = dw_data = odls = NULL;
+    PyArrayObject **g_data, **c_data, *mask,
+                  *g_start, *c_start, *c_dims, *g_dims, *dwa;
+    mask = g_start = c_start = c_dims = g_dims = NULL;
+    double *dls = NULL;
+    int refratio, ll, direction, n;
+    npy_int64 gxs, gys, gzs, gxe, gye, gze;
+    npy_int64 cxs, cys, czs, cxe, cye, cze;
+    npy_int64 ixs, iys, izs, ixe, iye, ize;
+    npy_int64 gxi, gyi, gzi, cxi, cyi, czi;
+    npy_int64 cdx, cdy, cdz;
+    npy_int64 dw[3];
+    int i, axis;
+    int ci, cj, ck, ri, rj, rk;
+    int total = 0;
+
+    if (!PyArg_ParseTuple(args, "iOOOOOOOOOi",
+            &refratio, &og_start, &oc_start,
+            &oc_data, &og_data,
+            &oc_dims, &og_dims, &omask, &dw_data, &odls, &axis))
+    return PyErr_Format(_dataCubeError,
+            "DataCubeGeneric: Invalid parameters.");
+
+    g_start = (PyArrayObject *) PyArray_FromAny(og_start,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(g_start == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: g_start invalid.");
+      goto _fail;
+    }
+
+    c_start = (PyArrayObject *) PyArray_FromAny(oc_start,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(c_start == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: c_start invalid.");
+      goto _fail;
+    }
+
+    g_dims  = (PyArrayObject *) PyArray_FromAny(og_dims,
+                PyArray_DescrFromType(NPY_INT32), 1, 1, 0, NULL);
+    if(g_dims == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: g_dims invalid.");
+      goto _fail;
+    }
+
+    c_dims  = (PyArrayObject *) PyArray_FromAny(oc_dims,
+                PyArray_DescrFromType(NPY_INT32), 1, 1, 0, NULL);
+    if(c_dims == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: c_dims invalid.");
+      goto _fail;
+    }
+
+    mask    = (PyArrayObject *) PyArray_FromAny(omask,
+                PyArray_DescrFromType(NPY_INT32), 3, 3, 0, NULL);
+    if(mask == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: mask invalid.");
+      goto _fail;
+    }
+
+    dwa     = (PyArrayObject *) PyArray_FromAny(dw_data,
+                PyArray_DescrFromType(NPY_INT64), 1, 1, 0, NULL);
+    if(dwa == NULL){
+      PyErr_Format(_dataCubeError, "FillRegion: domain width invalid.");
+      goto _fail;
+    }
+    for (i=0;i<3;i++)dw[i] = *(npy_int64*) PyArray_GETPTR1(dwa, i);
+
+    int n_fields = PyList_Size(oc_data);
+    if(n_fields == 0) {
+      PyErr_Format(_dataCubeError,
+          "CombineGrids: Length zero for c_data is invalid.");
+      goto _fail;
+    }
+    if (!PyList_Check(og_data) || (PyList_Size(og_data) != n_fields)){
+      PyErr_Format(_dataCubeError,
+          "CombineGrids: g_data must be a list of arrays same length as c_data!");
+      goto _fail;
+    }
+    if (!PyList_Check(odls) || (PyList_Size(odls) != n_fields)){
+      PyErr_Format(_dataCubeError,
+          "CombineGrids: dls must be a list of arrays same length as c_data!");
+      goto _fail;
+    }
+
+    c_data = (PyArrayObject**)
+             malloc(sizeof(PyArrayObject*)*n_fields);
+    g_data = (PyArrayObject**)
+             malloc(sizeof(PyArrayObject*)*n_fields);
+    dls = (double *) malloc(sizeof(double) * n_fields);
+    PyObject *temp = NULL;
+    for (n=0;n<n_fields;n++)c_data[n]=g_data[n]=NULL;
+    for (n=0;n<n_fields;n++){
+      /* Borrowed reference ... */
+      temp = PyList_GetItem(odls, n);
+      dls[n] = PyFloat_AsDouble(temp);
+
+      tg_data = (PyObject *) PyList_GetItem(og_data, n);
+      /* We set up an array so we only have to do this once 
+         Note that this is an array in the C sense, not the NumPy sense */
+      g_data[n]    = (PyArrayObject *) PyArray_FromAny(tg_data,
+          PyArray_DescrFromType(NPY_FLOAT64), 3, 3,
+          NPY_UPDATEIFCOPY, NULL);
+      if((g_data[n]==NULL) || (g_data[n]->nd != 3)) {
+        PyErr_Format(_dataCubeError,
+            "CombineGrids: Three dimensions required for g_data[%i].",n);
+        goto _fail;
+      }
+      tc_data = (PyObject *) PyList_GetItem(oc_data, n);
+      c_data[n]    = (PyArrayObject *) PyArray_FromAny(tc_data,
+          PyArray_DescrFromType(NPY_FLOAT64), 2, 2,
+          NPY_UPDATEIFCOPY, NULL);
+      if((c_data[n]==NULL) || (c_data[n]->nd != 2)) {
+        PyErr_Format(_dataCubeError,
+            "CombineGrids: Two dimensions required for c_data[%i].",n);
+        goto _fail;
+      }
+    }
+
+    /* g[xyz][se] are the start and end index in integers 
+       of the grid, at its refinement level               */
+    gxs = *(npy_int64 *) PyArray_GETPTR1(og_start, 0);
+    gys = *(npy_int64 *) PyArray_GETPTR1(og_start, 1);
+    gzs = *(npy_int64 *) PyArray_GETPTR1(og_start, 2);
+    gxe = gxs + *(npy_int32 *) PyArray_GETPTR1(og_dims, 0);
+    gye = gys + *(npy_int32 *) PyArray_GETPTR1(og_dims, 1);
+    gze = gzs + *(npy_int32 *) PyArray_GETPTR1(og_dims, 2);
+
+    /* c[xyz][se] are the start and end index in integers
+       of the covering grid, at its refinement level      */
+    cxs = *(npy_int64 *) PyArray_GETPTR1(oc_start, 0);
+    cys = *(npy_int64 *) PyArray_GETPTR1(oc_start, 1);
+    czs = *(npy_int64 *) PyArray_GETPTR1(oc_start, 2);
+
+    /* cd[xyz] are the dimensions of the covering grid */
+    cdx = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 0));
+    cdy = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 1));
+    cdz = (*(npy_int32 *) PyArray_GETPTR1(oc_dims, 2));
+    cxe = (cxs + cdx - 1);
+    cye = (cys + cdy - 1);
+    cze = (czs + cdz - 1);
+
+    /* It turns out that C89 doesn't define a mechanism for choosing the sign
+       of the remainder.
+    */
+    int x_loc, y_loc; // For access into the buffer
+    for(cxi=cxs;cxi<=cxe;cxi++) {
+        ci = (cxi % dw[0]);
+        ci = (ci < 0) ? ci + dw[0] : ci;
+        if ( ci < gxs*refratio || ci >= gxe*refratio) continue;
+        gxi = floor(ci / refratio) - gxs;
+        for(cyi=cys;cyi<=cye;cyi++) {
+            cj = cyi % dw[1];
+            cj = (cj < 0) ? cj + dw[1] : cj;
+            if ( cj < gys*refratio || cj >= gye*refratio) continue;
+            gyi = floor(cj / refratio) - gys;
+            for(czi=czs;czi<=cze;czi++) {
+                ck = czi % dw[2];
+                ck = (ck < 0) ? ck + dw[2] : ck;
+                if ( ck < gzs*refratio || ck >= gze*refratio) continue;
+                gzi = floor(ck / refratio) - gzs;
+                    if (*(npy_int32*)PyArray_GETPTR3(mask, gxi,gyi,gzi) > 0)
+                {
+                switch (axis) {
+                  case 0: x_loc = cyi-cys; y_loc = czi-czs; break;
+                  case 1: x_loc = cxi-cxs; y_loc = czi-czs; break;
+                  case 2: x_loc = cxi-cys; y_loc = cyi-cys; break;
+                }
+                for(n=0;n<n_fields;n++){
+                    *(npy_float64*) PyArray_GETPTR2(c_data[n], x_loc, y_loc)
+                    +=  *(npy_float64*) PyArray_GETPTR3(g_data[n], gxi, gyi, gzi) 
+                        * dls[n] / refratio;
+                }
+                total += 1;
+                }
+            }
+        }
+    }
+
+    Py_DECREF(g_start);
+    Py_DECREF(c_start);
+    Py_DECREF(g_dims);
+    Py_DECREF(c_dims);
+    Py_DECREF(mask);
+    for(n=0;n<n_fields;n++) {
+        Py_DECREF(g_data[n]);
+        Py_DECREF(c_data[n]);
+    }
+    if(dls!=NULL)free(dls);
+    if(g_data!=NULL)free(g_data);
+    if(c_data!=NULL)free(c_data);
+    PyObject *status = PyInt_FromLong(total);
+    return status;
+
+_fail:
+    Py_XDECREF(g_start);
+    Py_XDECREF(c_start);
+    Py_XDECREF(g_dims);
+    Py_XDECREF(c_dims);
+    Py_XDECREF(mask);
+    if(dls!=NULL)free(dls);
+    for(n=0;n<n_fields;n++) {
+        if(g_data!=NULL)if(g_data[n]!=NULL){Py_XDECREF(g_data[n]);}
+        if(c_data!=NULL)if(c_data[n]!=NULL){Py_XDECREF(c_data[n]);}
+    }
+    if(g_data!=NULL)free(g_data);
+    if(c_data!=NULL)free(c_data);
+    return NULL;
+}
+
 static PyObject *_findContoursError;
 
-npy_int64 process_neighbors(PyArrayObject*, npy_int64, npy_int64, npy_int64);
+int process_neighbors(PyArrayObject*, npy_int64, npy_int64, npy_int64,
+                            int first);
 static PyObject *
 Py_FindContours(PyObject *obj, PyObject *args)
 {
@@ -874,6 +1293,7 @@ Py_FindContours(PyObject *obj, PyObject *args)
     PyArrayObject *con_ids, *xi, *yi, *zi;
     xi=yi=zi=con_ids=NULL;
     npy_int64 i, j, k, n;
+    int status;
 
     i = 0;
     if (!PyArg_ParseTuple(args, "OOOO",
@@ -921,7 +1341,8 @@ Py_FindContours(PyObject *obj, PyObject *args)
       i=*(npy_int64 *)PyArray_GETPTR1(xi,n);
       j=*(npy_int64 *)PyArray_GETPTR1(yi,n);
       k=*(npy_int64 *)PyArray_GETPTR1(zi,n);
-      process_neighbors(con_ids, i, j, k);
+      status = process_neighbors(con_ids, i, j, k, 1);
+      if(status < 0) break;
     }
 
     Py_DECREF(con_ids);
@@ -929,8 +1350,8 @@ Py_FindContours(PyObject *obj, PyObject *args)
     Py_DECREF(yi);
     Py_DECREF(zi);
 
-    PyObject *status = PyInt_FromLong(1);
-    return status;
+    PyObject *retval = PyInt_FromLong(status);
+    return retval;
 
     _fail:
         Py_XDECREF(con_ids);
@@ -940,13 +1361,17 @@ Py_FindContours(PyObject *obj, PyObject *args)
         return NULL;
 }
 
-npy_int64 process_neighbors(PyArrayObject *con_ids, npy_int64 i, npy_int64 j, npy_int64 k)
+int process_neighbors(PyArrayObject *con_ids, npy_int64 i, npy_int64 j,
+                            npy_int64 k, int first)
 {
   npy_int64 off_i, off_j, off_k;
-  int spawn_check;
+  int spawn_check, status;
   int mi, mj, mk;
+  static int stack_depth;
+  if (first == 1) stack_depth = 0;
+  else stack_depth++;
+  if (stack_depth > 10000) return -1;
   npy_int64 *fd_off, *fd_ijk;
-  npy_int64 new_cid;
   mi = con_ids->dimensions[0];
   mj = con_ids->dimensions[1];
   mk = con_ids->dimensions[2];
@@ -966,13 +1391,15 @@ npy_int64 process_neighbors(PyArrayObject *con_ids, npy_int64 i, npy_int64 j, np
           }
           if(*fd_off < *fd_ijk){
             *fd_off = *fd_ijk;
-            new_cid = process_neighbors(con_ids, off_i, off_j, off_k);
-            if (new_cid != *fd_ijk) spawn_check += 1;
-            *fd_ijk = new_cid;
+            status = process_neighbors(con_ids, off_i, off_j, off_k, 0);
+            if (*fd_off != *fd_ijk) spawn_check += 1;
+            *fd_ijk = *fd_off;
+            if (status < 0) return -1;
           }
         }
   } while (spawn_check > 0);
-  return (npy_int64) *fd_ijk;
+  stack_depth -= 1;
+  return 1;
 }
 
 static PyObject *_interpolateError;
@@ -1253,6 +1680,8 @@ static PyMethodDef _combineMethods[] = {
     {"FindContours", Py_FindContours, METH_VARARGS},
     {"FindBindingEnergy", Py_FindBindingEnergy, METH_VARARGS},
     {"OutputFloatsToFile", Py_OutputFloatsToFile, METH_VARARGS},
+    {"FillRegion", Py_FillRegion, METH_VARARGS},
+    {"FillBuffer", Py_FillBuffer, METH_VARARGS},
     {NULL, NULL} /* Sentinel */
 };
 
