@@ -104,12 +104,12 @@ class RunParallelHOP(ParallelAnalysisInterface):
                 # Also test to see if the distance to this corner is within
                 # max_padding, which is more likely the case with load-balancing
                 # turned on.
-                dx = na.min( na.abs(my_vertex[0] - vertex[0]), \
-                    self.period[0] - na.abs(my_vertex[0] - vertex[0]))
-                dy = na.min( na.abs(my_vertex[1] - vertex[1]), \
-                    self.period[1] - na.abs(my_vertex[1] - vertex[1]))
-                dz = na.min( na.abs(my_vertex[2] - vertex[2]), \
-                    self.period[2] - na.abs(my_vertex[2] - vertex[2]))
+                dx = min( na.fabs(my_vertex[0] - vertex[0]), \
+                    self.period[0] - na.fabs(my_vertex[0] - vertex[0]))
+                dy = min( na.fabs(my_vertex[1] - vertex[1]), \
+                    self.period[1] - na.fabs(my_vertex[1] - vertex[1]))
+                dz = min( na.fabs(my_vertex[2] - vertex[2]), \
+                    self.period[2] - na.fabs(my_vertex[2] - vertex[2]))
                 d = na.sqrt(dx*dx + dy*dy + dz*dz)
                 if d <= self.max_padding:
                     self.neighbors.add(int(vertex[3]))
@@ -1374,7 +1374,7 @@ class RunParallelHOP(ParallelAnalysisInterface):
         max_radius = na.zeros(self.group_count, dtype='float64')
         if calc:
             com = self.CoM[subchain]
-            rad = na.abs(com - loc)
+            rad = na.fabs(com - loc)
             dist = (na.minimum(rad, self.period - rad)**2.).sum(axis=1)
             dist = dist[sort]
             for i, u in enumerate(uniq_subchain):
